#############################################################
#
# Linux kernel targets
#
# Note:  If you have any patches to apply, create the directory
# sources/kernel-patches and put your patches in there and number
# them in the order you wish to apply them...  i.e.
#
#   patches/kernel/001-my-special-stuff.bz2
#   patches/kernel/003-gcc-Os.bz2
#   patches/kernel/004_no-warnings.bz2
#   patches/kernel/030-lowlatency-mini.bz2
#   patches/kernel/031-lowlatency-fixes-5.bz2
#   patches/kernel/099-shutup.bz2
#   etc...
#
# these patches will all be applied by the patch-kernel.sh
# script (which will also abort the build if it finds rejects)
#  -Erik
#
# It will also do patch-o-matic patches, dependant on the patch-o-matic
# TARGET and the POM_PATCHES list in the main Makefile.
#
# -Thomas.
#
#############################################################

# Version of Linux to download and then apply patches to
DOWNLOAD_LINUX_VERSION=2.4.28
# Version of Linux AFTER patches
LINUX_VERSION=2.4.28-floppyfw-$(FLOPPYFW_VERSION)

LINUX_FORMAT=bzImage
LINUX_BINLOC=arch/$(ARCH)/boot/$(LINUX_FORMAT)

LINUX_DIR=$(BUILD_DIR)/linux-$(LINUX_VERSION)
LINUX_SOURCE=linux-$(DOWNLOAD_LINUX_VERSION).tar.bz2
LINUX_SITE=ftp://ftp.kernel.org/pub/linux/kernel/v2.4
LINUX_KCONFIG=$(CONFIG_DIR)/config-kernel
LINUX_KERNEL=$(LINUX_DIR)/vmlinuz
# Used by pcmcia-cs and others
LINUX_SOURCE_DIR=$(LINUX_DIR)

UPX=upx-1.11


$(DL_DIR)/$(LINUX_SOURCE): 
	 $(WGET) -P $(DL_DIR) $(LINUX_SITE)/$(LINUX_SOURCE)

$(LINUX_DIR)/.unpacked: $(DL_DIR)/$(LINUX_SOURCE) 
	-rm -rf $(MODULES_DIR)/lib
	-rm -f $(MODULES_DIR)/.stripped
	-rm -rf $(LINUX_DIR)
	bzcat $(DL_DIR)/$(LINUX_SOURCE) | tar -C $(BUILD_DIR) -xvf -
ifneq ($(DOWNLOAD_LINUX_VERSION),$(LINUX_VERSION))
	# Rename the dir from the downloaded version to the AFTER patch version	
	mv -f $(BUILD_DIR)/linux-$(DOWNLOAD_LINUX_VERSION) $(BUILD_DIR)/linux-$(LINUX_VERSION)
endif
	mkdir -p $(PATCH_DIR)/kernel
	# Patching the kernel.
	$(PATCH_DIR)/patch-kernel.sh $(LINUX_DIR) $(PATCH_DIR)/kernel

	# Patching the kernel with iptables-pom-stuff:
ifeq ($(filter $(TARGETS),patch-o-matic), patch-o-matic)
	(cd $(PATCH_O_MATIC_DIR); \
		KERNEL_DIR=$(LINUX_DIR) \
		IPTABLES_DIR=$(IPTABLES_BUILD_DIR) \
		./runme	--batch $(POM_PATCHES) \
		 );
endif

	# From a lwn article: need gcc 3.4/3.5
	#  -funit-at-a-time  -mregparm=3
	# From a patch by Eric Anderson: (But these don't reduce size, weird)
	#  -falign-functions=2 -falign-jumps=2 -falign-labels=2 -falign-loops=2
	$(SED) "s/\-O2/\-Os/;" $(LINUX_DIR)/Makefile
	-(cd $(BUILD_DIR); ln -sf $(LINUX_DIR) linux)
	touch $(LINUX_DIR)/.unpacked

$(LINUX_KCONFIG):
	@if [ ! -f "$(LINUX_KCONFIG)" ] ; then \
		echo ""; \
		echo "You should create a .config for your kernel"; \
		echo "and install it as $(LINUX_KCONFIG)"; \
		echo ""; \
		sleep 5; \
	fi;


$(LINUX_DIR)/.configured $(BUILD_DIR)/linux/.configured:  $(LINUX_DIR)/.unpacked  $(LINUX_KCONFIG)
	-cp $(LINUX_KCONFIG) $(LINUX_DIR)/.config
ifeq ($(strip $(EMULATE_FPU)),true)
	$(SED) "s/^.*CONFIG_MATH_EMULATION.*/CONFIG_MATH_EMULATION=y/;" \
		$(LINUX_DIR)/.config
else
	$(SED) "s/^.*CONFIG_MATH_EMULATION.*/CONFIG_MATH_EMULATION=n/;" \
		$(LINUX_DIR)/.config
endif
	$(SED) "s/^EXTRAVERSION.*/EXTRAVERSION=-floppyfw-$(FLOPPYFW_VERSION)/;" \
		$(LINUX_DIR)/Makefile
	$(MAKE) -C $(LINUX_DIR) oldconfig include/linux/version.h
	touch $(LINUX_DIR)/.configured

$(LINUX_DIR)/.depend_done:  $(LINUX_DIR)/.configured
	$(MAKE) -C $(LINUX_DIR) dep
	touch $(LINUX_DIR)/.depend_done

$(LINUX_DIR)/$(LINUX_BINLOC): $(LINUX_DIR)/.depend_done
	$(MAKE) -C $(LINUX_DIR) $(LINUX_FORMAT)
	$(MAKE) -C $(LINUX_DIR) modules

$(MODULES_DIR)/lib/modules/$(LINUX_VERSION)/modules.dep: $(LINUX_DIR)/.depend_done
	rm -rf $(MODULES_DIR)/lib/modules
	$(MAKE) -C $(LINUX_DIR) INSTALL_MOD_PATH=$(MODULES_DIR) modules_install
	(cd $(MODULES_BASE_DIR) ; cat $(MODULES_DIR)/lib/modules/$(LINUX_VERSION)/modules.dep | /$(BASE_DIR)/perl/capsfix.pl > $(TMP_DIR)/moddep)
	mv $(TMP_DIR)/moddep $(MODULES_DIR)/lib/modules/$(LINUX_VERSION)/modules.dep

	(cd $(MODULES_DIR)/lib/modules; ln -s $(LINUX_VERSION)/kernel/drivers .)

$(LINUX_KERNEL): $(LINUX_DIR)/$(LINUX_BINLOC)
	cp -fa $(LINUX_DIR)/$(LINUX_BINLOC) $(LINUX_KERNEL)
	cp -fa $(LINUX_DIR)/$(LINUX_BINLOC) $(LINUX_KERNEL).upx
	$(UPX) -9 $(LINUX_KERNEL).upx
	touch -c $(LINUX_KERNEL)

linux:  patch-o-matic $(IPTABLES_BUILD_DIR)/.configured $(LINUX_KERNEL) $(MODULES_DIR)/lib/modules/$(LINUX_VERSION)/modules.dep

linux-source: $(DL_DIR)/$(LINUX_SOURCE)

# This has been renamed so we do _NOT_ by default run this on 'make clean'
linuxclean: clean
	rm -f $(LINUX_KERNEL)
	-$(MAKE) -C $(LINUX_DIR) clean

linux-dirclean:
	rm -rf $(LINUX_DIR)
